==> daruma-py2compt-0.0.4.tar.gz: daruma-py2compt-0.0.4/daruma_py2compt/functions.py <==
import numpy as np
import sys, os
from datetime import datetime
import time
from struct import unpack


dir_name = os.path.dirname(__file__)
sys.path.append(dir_name)



def load_AAindex():
    key = ["A","C","D","E","F","G","H","I","K","L","M","N","P","Q","R","S","T","V","W","Y","X"]

    features_path = os.path.join(os.path.dirname(__file__), 'data', 'AAindex553-Normal-X0.feature')
    with open(features_path,"rb") as f:
        binary_data = f.read()
    data = unpack(">11613f",binary_data)

    Dic = {}
    for a,i in zip(key,range(0,11613,553)):
        Dic[a] = data[i:i+553]
    
    return Dic


##### smoothing処理
# smoothing
def smoothing(window,prob):
    L = len(prob)
    pool = window//2
    new_prob = list(map(mean,[prob[:i] for i in range(pool+1,window)]+[prob[i:i+window] for i in range(L-window)]+[prob[i:] for i in range(L-window,L-pool)]))
    return new_prob

def mean(l):
    return sum(l)/len(l)

# クラスわけ
def classify(prob,threshold=0.5):
    return np.where(np.array(prob)>=threshold,1,0).tolist()

# 10残基未満のIDRをNegativeに置換
def remove_short_idr(pred):
    pred = "".join(map(str,pred))
    pred = pred.replace("10","1-0").replace("01","0-1")
    return list(map(int,list("".join([_replace(region,"1","0") for region in pred.split("-")]))))

# 10残基未満のStructureをPositiveに置換
def remove_short_stru(pred):
    pred = "".join(map(str,pred))
    pred = pred.replace("10","1-0").replace("01","0-1")
    return list(map(int,list("".join([_replace(region,"0","1") for region in pred.split("-")]))))

def _replace(region,label1,label2):
    l = len(region)
    if l < 10:
        return region.replace(label1,label2)
    else:
        return region


##### layer

def relu(x):
    return np.maximum(0, x)

def softmax(x):
    x = x - np.max(x, axis=-1, keepdims=True)   # オーバーフロー対策
    return np.exp(x) / np.sum(np.exp(x), axis=-1, keepdims=True)


def D1im2col(x, filter_l):
    L, dim = x.shape
    out_h = L - filter_l + 1

    col = np.zeros((filter_l, dim, out_h),dtype="float32")

    for j in range(filter_l):
        j_max = j + out_h
        for i in range(dim):
            col[j, i, :] = x[j:j_max, i]

    col = col.transpose(2, 0, 1).reshape(out_h, -1)
    return col

class D1Conv:
    def __init__(self,W,b,active):
        FW,_,FN = W.shape #(FW,dim,FN)

        self.FW = FW
        self.FN = FN
        self.W = W.reshape(-1,FN)
        self.b = b
        self.active = active
        
    def forward(self,x):
        
        L,_ = x.shape
        OL = L - self.FW + 1

        col = D1im2col(x,self.FW)
        x = np.dot(col,self.W) + self.b
        x = x.reshape(OL,self.FN)

        return self.active(x)

class Affine:
    def __init__(self,W,b,active):
        self.W = W
        self.b = b
        self.active = active
        
    def forward(self,x):
        return self.active(np.dot(x,self.W) + self.b)



##### result file

class ResultFileManager:
    def __init__(self,output_path):
        self.path = output_path

    def append_write(self):
        pass

    def close_manager(self):
        pass


class format_residue_single(ResultFileManager):
    def __init__(self,output_path="."):
        if not output_path.endswith("/"):
            output_path += "/"

        if not os.path.isdir(output_path):
            print("The directory output_path does not exist.")
        
        
        self.path = output_path + "disorder/"
        if not os.path.exists(self.path):
            os.makedirs(self.path)

    def append_write(self,ac,seq,pred_prob,pred_class):
        with open(self.path+ac+".daruma","w") as f:
            out = "\n".join(["{} {} {:.4f} {}".format(i,res,p,c) for i,res,p,c in zip(range(1,len(seq)+1),seq,pred_prob,pred_class)])
            f.write(">{}\n{}\n".format(ac,out))


class format_residue_multi(ResultFileManager):
    def __init__(self,output_path):
        super().__init__(output_path)

        with open(self.path,"w") as f:
            f.write("")

    def append_write(self,ac,seq,pred_prob,pred_class):
        with open(self.path,"a") as f:
            out = "\n".join(["{} {} {:.4f} {}".format(i,res,p,c) for i,res,p,c in zip(range(1,len(seq)+1),seq,pred_prob,pred_class)])
            f.write(">{}\n{}\n".format(ac,out))


class format_4lines(ResultFileManager):
    def __init__(self,output_path):
        super().__init__(output_path)

        with open(self.path,"w") as f:
            f.write("")
    
    def append_write(self,ac,seq,pred_prob,pred_class):
        with open(self.path,"a") as f:
           f.write(f'>{ac}\n{",".join(seq)}\n{",".join(map(lambda x:"{:.4f}".format(x),pred_prob))}\n{",".join(map(str,pred_class))}\n')

==> daruma-py2compt-0.0.5.tar.gz: daruma-py2compt-0.0.5/daruma_py2compt/functions.py <==
import numpy as np
import sys, os
from datetime import datetime
import time
from struct import unpack


dir_name = os.path.dirname(__file__)
sys.path.append(dir_name)



def load_AAindex():
    key = ["A","C","D","E","F","G","H","I","K","L","M","N","P","Q","R","S","T","V","W","Y","X"]

    features_path = os.path.join(os.path.dirname(__file__), 'data', 'AAindex553-Normal-X0.feature')
    with open(features_path,"rb") as f:
        binary_data = f.read()
    data = unpack(">11613f",binary_data)

    Dic = {}
    for a,i in zip(key,range(0,11613,553)):
        Dic[a] = data[i:i+553]
    
    return Dic


def smoothing(window,prob):
    L = len(prob)
    pool = window//2
    new_prob = list(map(mean,[prob[:i] for i in range(pool+1,window)]+[prob[i:i+window] for i in range(L-window)]+[prob[i:] for i in range(L-window,L-pool)]))
    return new_prob

def mean(l):
    return sum(l)/len(l)


def classify(prob,threshold=0.5):
    return np.where(np.array(prob)>=threshold,1,0).tolist()


def remove_short_idr(pred):
    pred = "".join(map(str,pred))
    pred = pred.replace("10","1-0").replace("01","0-1")
    return list(map(int,list("".join([_replace(region,"1","0") for region in pred.split("-")]))))


def remove_short_stru(pred):
    pred = "".join(map(str,pred))
    pred = pred.replace("10","1-0").replace("01","0-1")
    return list(map(int,list("".join([_replace(region,"0","1") for region in pred.split("-")]))))

def _replace(region,label1,label2):
    l = len(region)
    if l < 10:
        return region.replace(label1,label2)
    else:
        return region



def relu(x):
    return np.maximum(0, x)

def softmax(x):
    x = x - np.max(x, axis=-1, keepdims=True)
    return np.exp(x) / np.sum(np.exp(x), axis=-1, keepdims=True)


def D1im2col(x, filter_l):
    L, dim = x.shape
    out_h = L - filter_l + 1

    col = np.zeros((filter_l, dim, out_h),dtype="float32")

    for j in range(filter_l):
        j_max = j + out_h
        for i in range(dim):
            col[j, i, :] = x[j:j_max, i]

    col = col.transpose(2, 0, 1).reshape(out_h, -1)
    return col

class D1Conv:
    def __init__(self,W,b,active):
        FW,_,FN = W.shape

        self.FW = FW
        self.FN = FN
        self.W = W.reshape(-1,FN)
        self.b = b
        self.active = active
        
    def forward(self,x):
        
        L,_ = x.shape
        OL = L - self.FW + 1

        col = D1im2col(x,self.FW)
        x = np.dot(col,self.W) + self.b
        x = x.reshape(OL,self.FN)

        return self.active(x)

class Affine:
    def __init__(self,W,b,active):
        self.W = W
        self.b = b
        self.active = active
        
    def forward(self,x):
        return self.active(np.dot(x,self.W) + self.b)




class ResultFileManager:
    def __init__(self,output_path):
        self.path = output_path

    def append_write(self):
        pass

    def close_manager(self):
        pass


class format_residue_single(ResultFileManager):
    def __init__(self,output_path="."):
        if not output_path.endswith("/"):
            output_path += "/"

        if not os.path.isdir(output_path):
            print("The directory output_path does not exist.")
        
        
        self.path = output_path + "disorder/"
        if not os.path.exists(self.path):
            os.makedirs(self.path)

    def append_write(self,ac,seq,pred_prob,pred_class):
        with open(self.path+ac+".daruma","w") as f:
            out = "\n".join(["{} {} {:.4f} {}".format(i,res,p,c) for i,res,p,c in zip(range(1,len(seq)+1),seq,pred_prob,pred_class)])
            f.write(">{}\n{}\n".format(ac,out))


class format_residue_multi(ResultFileManager):
    def __init__(self,output_path):
        super().__init__(output_path)

        with open(self.path,"w") as f:
            f.write("")

    def append_write(self,ac,seq,pred_prob,pred_class):
        with open(self.path,"a") as f:
            out = "\n".join(["{} {} {:.4f} {}".format(i,res,p,c) for i,res,p,c in zip(range(1,len(seq)+1),seq,pred_prob,pred_class)])
            f.write(">{}\n{}\n".format(ac,out))


class format_4lines(ResultFileManager):
    def __init__(self,output_path):
        super().__init__(output_path)

        with open(self.path,"w") as f:
            f.write("")
    
    def append_write(self,ac,seq,pred_prob,pred_class):
        with open(self.path,"a") as f:
           f.write(f'>{ac}\n{",".join(seq)}\n{",".join(map(lambda x:"{:.4f}".format(x),pred_prob))}\n{",".join(map(str,pred_class))}\n')
